#include <Arduino.h>
#include <WiFiManager.h>
#include <WiFi.h>
#include <HTTPClient.h>
#include <PrettyOTA.h>

#define BOARDFIRMWARE "2.2"

#define BUTTONPIN 22
#define DEBOUNCEMS 15
#define BOARD "ESP32"
#define SERVICE "LEDsActionButton"
#define STATUSINTERVAL 600000

String actionURL = "http://moon.basement.lan/l/actionbutton.php?state=";
String statusURL = "http://moon.basement.lan/l/boardstatus.php";
unsigned short buttonState = HIGH, lastButtonState = HIGH;
unsigned long lastStatusTime = 0;

AsyncWebServer server(80);
PrettyOTA OTAUpdates;

void sendActionMessage(bool state) {
  Serial.println("Action button triggered");
  char webMsg[255] = "";
  strcat(webMsg, actionURL.c_str());
  strcat(webMsg, (state) ? "true" : "false");
  HTTPClient http;
  http.begin(webMsg);
  http.GET();
  http.end();
}

unsigned long sendDeviceStatus() {
  Serial.println("Sending device status");
  WiFiClient client;
  HTTPClient http;
  http.begin(client, statusURL.c_str());
  http.addHeader("Content-Type", "application/x-www-form-urlencoded");
  String httpData = "mac=" + WiFi.macAddress() + "&board=" + BOARD + "&firmware=" + BOARDFIRMWARE + "&service=" + SERVICE;
  int httpResponseCode = http.POST(httpData);
  http.end();
  return millis();
}

// Gets called when update starts
// updateMode can be FILESYSTEM or FIRMWARE
void OnOTAStart(NSPrettyOTA::UPDATE_MODE updateMode) {
  Serial.println("OTA update started");
  if(updateMode == NSPrettyOTA::UPDATE_MODE::FIRMWARE)
    Serial.println("Mode: Firmware");
  else if(updateMode == NSPrettyOTA::UPDATE_MODE::FILESYSTEM)
    Serial.println("Mode: Filesystem");
}

// Gets called while update is running
// currentSize: Number of bytes already processed
// totalSize: Total size of new firmware in bytes
void OnOTAProgress(uint32_t currentSize, uint32_t totalSize) {
  Serial.printf("OTA Progress Current: %u bytes, Total: %u bytes\n", currentSize, totalSize);
}

// Gets called when update finishes
void OnOTAEnd(bool successful) {
  if (successful)
    Serial.println("OTA update finished successfully");
  else
    Serial.println("OTA update failed");
}

void setup() {
  Serial.begin(115200);
  pinMode(BUTTONPIN, INPUT_PULLUP);
  Serial.println("MAC Address: " + WiFi.macAddress());
  WiFiManager wifiManager;
  wifiManager.autoConnect("LEDsActionButton");
  lastStatusTime = sendDeviceStatus();
  OTAUpdates.Begin(&server);
  OTAUpdates.SetHardwareID(SERVICE);
  OTAUpdates.OverwriteAppVersion(BOARDFIRMWARE);
  PRETTY_OTA_SET_CURRENT_BUILD_TIME_AND_DATE();
  OTAUpdates.OnStart(OnOTAStart);
  OTAUpdates.OnProgress(OnOTAProgress);
  OTAUpdates.OnEnd(OnOTAEnd);
  server.begin();
}

unsigned long lastDebounceTime = 0;
bool actionState = false;
bool debouncing = false;
void loop() {
  if ( millis() > (lastStatusTime + STATUSINTERVAL) ) {
    lastStatusTime = sendDeviceStatus();
  }
  if ( millis() > 86400000 ) {
    ESP.restart();
  }
  unsigned short buttonCheck = digitalRead(BUTTONPIN);
  if ( buttonCheck != lastButtonState ) {
    debouncing = true;
    lastDebounceTime = millis();
  }
  if ( debouncing && (millis() >= (lastDebounceTime + DEBOUNCEMS)) ) {
    debouncing = false;
    if ( buttonCheck != buttonState ) {
      buttonState = buttonCheck;
      if ( buttonState == LOW ) {
        actionState = !actionState;
        sendActionMessage(actionState);
      }
    }
  }
  lastButtonState = buttonCheck;
}
